//
//  ContextMenu+AmityMessagesTableViewController.swift
//  SampleApp
//
//  Created by Nishan on 12/10/2566 BE.
//  Copyright © 2566 BE David Zhang. All rights reserved.
//

import Foundation
import UIKit
import AmitySDK
import SwiftUI

extension AmityMessagesTableViewController {
    
    override func tableView(_ tableView: UITableView, contextMenuConfigurationForRowAt indexPath: IndexPath, point: CGPoint) -> UIContextMenuConfiguration? {
        // Get Message
        guard let message = getMessage(for: indexPath), !message.isDeleted else { return nil }
        
        let configuration = UIContextMenuConfiguration(identifier: nil, previewProvider: nil) { actions in
            
            // Default Menu
            let defaultMenu = self.createDefaultMenu(for: message)
            
            // Reactions
            let reactionMenu = self.createReactionMenu(for: message)
            
            // Attachment Options
            let mediaMenu = self.createMediaMenu(for: message)
            
            // Marker
            let eventMenu = self.createEventsMenu(for: message)
            
            // Misc Menu
            let misMenu = self.createMiscMenu(for: message)
            
            var allElements = [UIMenuElement]()
            allElements.append(contentsOf: defaultMenu)
            allElements.append(reactionMenu)
            allElements.append(mediaMenu)
            allElements.append(eventMenu)
            allElements.append(misMenu)
                        
            return UIMenu(title: "", children: allElements)
        }
        
        return configuration
    }
    
    func createMediaMenu(for message: AmityMessage) -> UIMenu {
        var options = [UIAction]()
        
        if message.messageType == .audio {
            let playAudio = UIAction(title: "Play Audio", image: nil) { action in
                self.playAudioFromMessage(message: message)
            }
            
            options.append(playAudio)
        }
        
        if message.messageType == .file {
            let downloadFile = UIAction(title: "Download File", image: nil) { action in
                self.downloadAndViewFileFromMessage(message: message)
            }
            
            options.append(downloadFile)
        }
        
        if message.messageType == .video {
            let watchVideo = UIAction(title: "Watch Video", image: nil) { action in
                guard let videoInfo = message.getVideoInfo() else { return }
                self.addVideoAlertAction(videoInfo: videoInfo)
            }
            
            options.append(watchVideo)
        }
        
        if let fileId = message.fileId {
            var fileURL: String
            if message.messageType == .image {
                let data = message.getImageInfo()
                fileURL = data?.fileURL ?? ""
            } else {
                let data = message.getFileInfo()
                fileURL = data?.fileURL ?? ""
            }
            
            let copyFileId = UIAction(title: "Copy File Id", image: nil) { action in
                UIPasteboard.general.string = fileId
            }
            
            let openFileURL = UIAction(title: "Open File URL", image: nil) { action in
                if let url = URL(string: fileURL) {
                    UIApplication.shared.open(url)
                }
            }
            
            let deleteFile = UIAction(title: "Delete File", image: nil) { action in
                Task { @MainActor in
                    let alertMessage: String
                    do {
                        let _ = try await self.fileRepo.deleteFile(fileId: fileId)
                        alertMessage = "File successfully deleted"
                    } catch {
                        alertMessage = "Error while deleting file"
                    }
                    self.showAlert(message: alertMessage)
                }
            }
            
            options.append(copyFileId)
            options.append(openFileURL)
            options.append(deleteFile)
        }
        
        let menu = UIMenu(title: "Attached Media", children: options)
        
        return menu
    }
    
    func createDefaultMenu(for message: AmityMessage) -> [UIMenuElement] {
        // Default Menu
        let updateAction = UIAction(title: "Update", image: UIImage(systemName: "square.and.pencil")) { action in
            self.updateAction(forMessage: message)
        }
        
        let deleteAction = UIAction(title: "Delete", image: UIImage(systemName: "trash")) { action in
            self.deleteAction(forMessage: message)
        }
        
        let deleteFailedAction = UIAction(title: "Delete Failed message", image: UIImage(systemName: "trash")) { action in
            self.deleteFailedMessage(message: message)
        }
        
        let forwardAction = UIAction(title: "Forward", image: UIImage(systemName: "arrow.up.forward")) { action in
            self.forwardMessage(message: message)
        }
        
        var actions = [updateAction, deleteAction, deleteFailedAction, forwardAction]
        
        if let parentId = message.parentId {
            let jumpMessage = UIAction(title: "Jump to Parent", image: UIImage(systemName: "app.connected.to.app.below.fill")) { action in
                self.jumpToParentMessage(parentId: parentId)
            }
            
            actions.append(jumpMessage)
        }
        
        return actions
    }
    
    func createReactionMenu(for message: AmityMessage) -> UIMenu {
        let messageId = message.messageId
        
        let addReaction = UIAction(title: "Add Reaction", image: nil) { action in
            self.addReactionAction(forMessage: message)
        }
        
        let removeReaction = UIAction(title: "Remove Reaction", image: nil) { action in
            self.removeReactionAction(forMessage: message)
        }
        
        let showMyReaction = UIAction(title: "Show My Reaction", image: nil) { action in
            self.showMyReactionAction(forMessage: message)
        }
        
        let showAllReaction = UIAction(title: "Show All Reactions", image: nil) { action in
            self.showAllReactions(forMessage: message)
        }
        
        let testReaction = UIAction(title: "Test Reaction", image: nil) { action in
            self.showReactionTestView(messageId: messageId)
        }
        
        let menu = UIMenu(title: "Reaction", children: [
            addReaction,
            removeReaction,
            showMyReaction,
            showAllReaction,
            testReaction
        ])
        
        return menu
    }
    
    func createEventsMenu(for message: AmityMessage) -> UIMenu {
        let observeEvents = UIAction(title: "View Real Time Events", image: nil) { action in
            self.showRealTimeEventObserverScreen(for: message)
        }
        
        let markAsDelivered = UIAction(title: "Mark As Delivered", image: nil) { action in
            self.markMessageAsDelivered(message: message)
        }
        
        let readDeliveredList = UIAction(title: "Read/Delivered List", image: nil) { action in
            self.showReadDeliveredList(message: message)
        }
        
        let menu = UIMenu(title: "Marker & RTE", children: [
            observeEvents,
            markAsDelivered,
            readDeliveredList
        ])
        
        return menu
    }
    
    func createMiscMenu(for message: AmityMessage) -> UIMenu {
        
        let copyMessageId = UIAction(title: "Copy Message Id", image: UIImage(systemName: "doc.on.doc")) { action in
            UIPasteboard.general.string = message.messageId
        }
        
        let printDetails = UIAction(title: "View Details", image: UIImage(systemName: "doc.plaintext")) { action in
            self.showMessageDetails(details: message.modelDescription)
        }
        
        let menu = UIMenu(title: "Miscellaneous", children: [
           copyMessageId,
           printDetails
        ])
        
        return menu
    }
}
