//
//  CommunityView.swift
//  SampleApp
//
//  Created by Michael Abadi on 10/07/20.
//  Copyright © 2020 David Zhang. All rights reserved.
//

import SwiftUI
import AmitySDK

/// Using live collection from view model.
struct CommunityView: View {
    
    @ObservedObject var viewModel: CommunityListViewModel
    
    @Environment(\.navigationController) var navigationController
    
    @State private var memberType = AmityCommunityQueryFilter.all
    @State private var sortType = AmityCommunitySortOption.lastCreated
    @State private var includeDeleted = false
    @State private var categoryId = ""
    
    @State private var showCategoryListScreen = false
    @State private var showCreateScreen = false
    @State private var forceScrollToTop = false
    @State private var selection: String?
    
    @State private var shouldShowCommunities: Bool = false
    
    var body: some View {
        VStack {
            if viewModel.type == .normal {
                // NOTE: Only show the query menu for `.normal`.
                // The user need to press "Get Communities" button manually.
                // For other types, the communities will be queried automatically.
                // See Also: .onAppear method in this view.
                CommunitiesQueryMenu
            }
            
            if shouldShowCommunities {
                CommunityCollectionView(communityCollection: viewModel.communityCollection, viewModel: viewModel)
            }
            
            Spacer()
        }
        .navigationBarTitle(Text(viewModel.pageTitle), displayMode: .inline)
        .toolbar {
            ToolbarItemGroup(placement: .navigationBarTrailing) {
                // Categories list
                Button(action: {
                    self.showCategoryListScreen = true
                }, label: {
                    Image(systemName: "list.bullet")
                })
                .sheet(isPresented: $showCategoryListScreen) {
                    CommunityCategoryListView()
                }
                // Create a new community
                NavigationLink(destination: CommunityEditorView(viewModel: CommunityEditorViewModel()), isActive: $showCreateScreen) {
                    Button(action: {
                        self.showCreateScreen = true
                    }, label: {
                        Image(systemName: "plus")
                    })
                }
            }
        }
        .onAppear(perform: {
            selection = nil
            showCreateScreen = false
            
            UITableViewCell.appearance().selectionStyle = .none
            UITableView.appearance().separatorStyle = .none
            UITextField.appearance().clearButtonMode = .always
        })
        
    }
    
    @ViewBuilder
    var CommunitiesQueryMenu: some View {
        VStack(alignment: .leading) {
            // Member Type
            HStack {
                Text("Member Type")
                    .font(.caption)
                Spacer()
                Picker("Member Type", selection: $memberType) {
                    Text("All").tag(AmityCommunityQueryFilter.all)
                    Text("Member").tag(AmityCommunityQueryFilter.userIsMember)
                    Text("Not Member").tag(AmityCommunityQueryFilter.userIsNotMember)
                }
                .pickerStyle(.segmented)
            }

            // Sort Type
            HStack {
                Text("Sort By")
                    .font(.caption)
                Spacer()
                Picker("Sort Type", selection: $sortType) {
                    Text("Last created").tag(AmityCommunitySortOption.lastCreated)
                    Text("First created").tag(AmityCommunitySortOption.firstCreated)
                    Text("Display Name").tag(AmityCommunitySortOption.displayName)
                }
                .pickerStyle(.segmented)
            }
            // Include Deleted
            HStack {
                Text("Include Deleted").font(.caption)
                Spacer()
                Picker("isDeleted", selection: $includeDeleted) {
                    Text("NO").tag(false)
                    Text("YES").tag(true)
                }.pickerStyle(.segmented)
            }
            
            // Search Field
            HStack {
                VStack(alignment: .leading) {
                    Text("CategoryId")
                        .font(.caption)
                    TextField("Search by category id", text: $categoryId)
                        .textFieldStyle(RoundedBorderTextFieldStyle())
                        .font(.callout)
                }
                
                VStack(alignment: .leading) {
                    Text("Keyword")
                        .font(.caption)
                    TextField("Search by display name", text: $viewModel.searchKeyword)
                        .textFieldStyle(RoundedBorderTextFieldStyle())
                        .font(.callout)
                }
            }
            
            // Query Button
            HStack {
                Spacer()
                
                Button("Get Communities") {
                    forceScrollToTop.toggle()
                    viewModel.setFilter(memberType)
                    viewModel.setSort(sortType)
                    viewModel.setIncludeDeleted(includeDeleted)
                    viewModel.setCategoryId(categoryId)
                    viewModel.queryCommunity()
                    
                    shouldShowCommunities = true
                    
                }.buttonStyle(.bordered)
                
                Spacer()
            }
        }
        .padding()
    }
}

struct CommunityCollectionView: View {

    @ObservedObject var communityCollection: AmityCollection<AmityCommunity>
    let viewModel: CommunityListViewModel
    
    @Environment(\.navigationController) var navigationController
    @State private var selection: String?
    @State var storyView: StoryListView?
    @State var showCheckPermissionAlert: Bool = false
        
    @State var enteredPermission: String = ""
    @State var permissionStatus: String = ""
    
    var body: some View {
        List(communityCollection.snapshots, id: \.communityId) { item in
            let community = CommunityModel(community: item)
            communityView(community)
        }
        .listStyle(.plain)
    }
    
    func communityView(_ community: CommunityModel) -> some View {
        ZStack {
            CommunityItemView(model: community, joinAction: {
                self.viewModel.joinCommunity(for: community, completion: nil)
            }, leaveAction: {
                self.viewModel.leaveCommunity(for: community, completion: nil)
            }, deleteAction: {
                self.viewModel.deleteCommunity(for: community, completion: nil)
            }, detailsAction: {
                self.selection = community.id + "+detail"
            }, updateAction: {
                self.selection = community.id + "+edit"
            }, membershipAction: {
                self.selection = community.id + "+participation"
            }, feedAction: {
                self.selection = community.id + "+feed"
                
                if selection == (community.id + "+feed") {
                    let feedManager = UserPostsFeedManager(client: AmityManager.shared.client!, userId: nil, userName: nil, community: community)
                    feedManager.feedType = .community
                    feedManager.community = community
                    
                    let postsFeedStoryboard = UIStoryboard(name: "Feed", bundle: nil)
                    let postsFeedController = postsFeedStoryboard.instantiateViewController(withIdentifier: UserPostsFeedViewController.identifier) as! UserPostsFeedViewController
                    postsFeedController.feedManager = feedManager
                    self.navigationController?.pushViewController(postsFeedController, animated: true)
                }
            }, viewStoryAction: {
                storyView = StoryListView(viewModel: StoryListViewModel(targetId: community.id))

                self.selection = community.id + "+story"
            }, realTimeEventAction: {
                let eventController = CommunityRealTimeEventController()
                let manager = CommunityRealTimeEventManager(community: community.communityObject)
                eventController.manager = manager
                
                self.navigationController?.pushViewController(eventController, animated: true)
            }, checkPermissionAction: {
                showCheckPermissionAlert = true
            }, copyCommunityIdAction: {
                UIPasteboard.general.string = community.communityId
            })
            .onAppear {
                if communityCollection.snapshots.last?.communityId == community.communityId {
                    self.viewModel.fetchNextPage()
                }
            }
            .sheet(isPresented: $showCheckPermissionAlert, content: {
                CheckPermissionView(permissionType: .community(id: community.id))
            })
            
            // Update Navigattion
            let editorView = CommunityEditorView(viewModel: CommunityEditorViewModel(draft: CommunityDraft(community: community)))
            
            NavigationLink(
                destination: editorView,
                tag: community.id + "+edit",
                selection: self.$selection,
                label: {
                    EmptyView()
                })
            .buttonStyle(PlainButtonStyle())
            .hidden()
        
            
            NavigationLink(
                destination: storyView?.environment(\.navigationController, navigationController),
                tag: community.id + "+story",
                selection: self.$selection,
                label: {
                    EmptyView()
                })
            .buttonStyle(PlainButtonStyle())
            .hidden()
            
            // Participation Navigation
            let participationModel = CommunityParticipationViewModel(membership: community.getCommunityObject().membership)
            NavigationLink(
                destination: CommunityParticipationView(viewModel: participationModel),
                tag: community.id + "+participation",
                selection: self.$selection,
                label: {
                    EmptyView()
                })
            .buttonStyle(PlainButtonStyle())
            .hidden()
            
            // Details
            NavigationLink(
                destination: CommunityDetailView(viewModel: CommunityDetailViewModel(community: community)),
                tag: community.id + "+detail",
                selection: self.$selection,
                label: {
                    EmptyView()
                })
            .buttonStyle(PlainButtonStyle())
            .hidden()
                        
        }
    }
}
